% Replicates the empirical results in "The empirical content of games with bounded regressors" by Brendan Kline

clear variables
clearvars -global
close all

RandStream.setGlobalStream(RandStream('mt19937ar','Seed', 123456));

alpha_test = 0.10;	
est_var_int = 1;
truncCI = @(x) [x(1) min(x(2),0)];

M = load('airline_smallmarkets.dat');
x1 = M(:,1);
x2 = M(:,2);
y00 = (M(:,3)==0).*(M(:,4)==0);
y01 = (M(:,3)==0).*(M(:,4)==1);
y10 = (M(:,3)==1).*(M(:,4)==0);
y11 = (M(:,3)==1).*(M(:,4)==1);
N = size(M(:,1),1)
mean([y00 y01 y10 y11])

h_int00 = 0.0128*N^(-1/16);
h_int11 = 0.0128*N^(-1/16);
[alphahat deltahat conf_alpha1 conf_alpha2 conf_delta1 conf_delta2 asymp_covar_int] = estimator_airlines(x1,x2,y00,y11,h_int00,h_int11,alpha_test,est_var_int);
conf_delta1 = truncCI(conf_delta1);
conf_delta2 = truncCI(conf_delta2);

fprintf(1, '$\\alpha_{LCC}$ & %3.4f & [%3.4f, %3.4f] \\\\ \n', alphahat(1), conf_alpha1);
fprintf(1, '$\\alpha_{OA}$ & %3.4f & [%3.4f, %3.4f] \\\\ \n', alphahat(2), conf_alpha2);
fprintf(1, '$\\Delta_{LCC}$ & %3.4f & [%3.4f, %3.4f] \\\\ \n', deltahat(1), conf_delta1);
fprintf(1, '$\\Delta_{OA}$ & %3.4f & [%3.4f, %3.4f] \\\\ \n', deltahat(2), conf_delta2);

