function [alphahat deltahat conf_alpha1 conf_alpha2 conf_delta1 conf_delta2 asymb_covar_int] = estimator_airlines(x1,x2,y00,y11,h_int00,h_int11,alpha_test,est_var_int)
	
	N = size(y00,1);

	beta1hat = [1];
	beta2hat = [1];

	%% Estimate the intercept parameters
		c_hat(1:N,1:2) = NaN;
		c_hat(:,1) = -x1*beta1hat';
		c_hat(:,2) = -x2*beta2hat';
		second_deriv_h_int00 = h_int00;
		second_deriv_h_int11 = h_int11;

		opt_fn_00 = @(x) -1*second_deriv_int((bsxfun(@minus, [x(1) x(2)], c_hat))/second_deriv_h_int00,y00,second_deriv_h_int00);
		opt_fn_11 = @(x) -1*second_deriv_int((bsxfun(@minus, [x(1) x(2)], c_hat))/second_deriv_h_int11,y11,second_deriv_h_int11);

		%est_bnd is used to avoid using the kernel in places where there are effectively no observations  
		est_bnd11 = quantile(c_hat(:,1),0.05);
		est_bnd12 = quantile(c_hat(:,2),0.05);
		est_bnd21 = quantile(c_hat(:,1),0.95);
		est_bnd22 = quantile(c_hat(:,2),0.95);

		opt = optimset('Display','none','TolX', 10e-6, 'Algorithm', 'interior-point','GradObj','off');
		fval1best = 1;
		fval2best = 1;

		for rep = 1:500,
			startpt1(1) = 1.3*(rand()-0.5);
			startpt1(2) = 1.3*(rand()-0.5);
			[alphahat1{rep}, fval1{rep}] = fmincon(opt_fn_00,startpt1, [], [], [], [], [est_bnd11 est_bnd12],[est_bnd21 est_bnd22],[],opt);
			if fval1{rep} < fval1best,
				alphahat = alphahat1{rep};
			end;
		end;
		for rep = 1:500,
			startpt2(1) = -rand()-0.5;
			startpt2(2) = -rand()-0.5;
			[alphadeltahat2{rep}, fval2{rep}] = fmincon(opt_fn_11,alphahat + startpt2, [], [], [], [], [est_bnd11 est_bnd12],[min(est_bnd21,alphahat(1)) min(est_bnd22,alphahat(2))],[],opt);
			if fval2{rep} < fval2best,
				alphadeltahat = alphadeltahat2{rep};
			end;
		end;

		deltahat = alphadeltahat - alphahat;
			
		%% Estimate the variance of the estimate of the intercept parameters --- var_int_cons depends on the kernel
		if est_var_int,
			C = [1 0 0 0; 0 1 0 0; -1 0 1 0; 0 -1 0 1];
			var_int_cons = 0.583426;
			
			fourth_deriv_h_int00 = h_int00 * (N^(-1/18)/N^(-1/16));
			mean_h_int00 = h_int00 * (N^(-1/10)/N^(-1/16));
			fourth_deriv_h_int11 = h_int11 * (N^(-1/18)/N^(-1/16));
			mean_h_int11 = h_int11 * (N^(-1/10)/N^(-1/16));
			density_h_int = h_int11 * (N^(-1/10)/N^(-1/16));

			alphahatdiff_chat = bsxfun(@minus, alphahat, c_hat);
			alphadeltahatdiff_chat = bsxfun(@minus, alphadeltahat, c_hat);

			d2dgdg(1,1) = fourth_deriv_int((alphahatdiff_chat)/fourth_deriv_h_int00,y00,fourth_deriv_h_int00, 1, 2, 1, 1);
			d2dgdg(1,2) = fourth_deriv_int((alphahatdiff_chat)/fourth_deriv_h_int00,y00,fourth_deriv_h_int00, 1, 2, 1, 2);
			d2dgdg(2,1) = fourth_deriv_int((alphahatdiff_chat)/fourth_deriv_h_int00,y00,fourth_deriv_h_int00, 1, 2, 2, 1);
			d2dgdg(2,2) = fourth_deriv_int((alphahatdiff_chat)/fourth_deriv_h_int00,y00,fourth_deriv_h_int00, 1, 2, 2, 2);
			d2dgdg(3,3) = fourth_deriv_int((alphadeltahatdiff_chat)/fourth_deriv_h_int11,y11,fourth_deriv_h_int11, 1, 2, 1, 1);
			d2dgdg(3,4) = fourth_deriv_int((alphadeltahatdiff_chat)/fourth_deriv_h_int11,y11,fourth_deriv_h_int11, 1, 2, 1, 2);
			d2dgdg(4,3) = fourth_deriv_int((alphadeltahatdiff_chat)/fourth_deriv_h_int11,y11,fourth_deriv_h_int11, 1, 2, 2, 1);
			d2dgdg(4,4) = fourth_deriv_int((alphadeltahatdiff_chat)/fourth_deriv_h_int11,y11,fourth_deriv_h_int11, 1, 2, 2, 2);
			d2dgdg(1:2,3:4) = 0;
			d2dgdg(3:4,1:2) = 0;
			invd2dgdg = inv(d2dgdg);
			
			Omega(1,1) = var_int_cons;
			Omega(2,2) = var_int_cons;
			Omega(1:2,1:2) = (Omega(1:2,1:2).*mean_int((alphahatdiff_chat)/mean_h_int00,y00,mean_h_int00)*(1-mean_int((alphahatdiff_chat)/mean_h_int00,y00,mean_h_int00)))./density_int((alphahatdiff_chat)/density_h_int,density_h_int);
			Omega(3,3) = var_int_cons;
			Omega(4,4) = var_int_cons;
			Omega(3:4,3:4) = (h_int00/h_int11)^8*(Omega(3:4,3:4).*mean_int((alphadeltahatdiff_chat)/mean_h_int11,y11,mean_h_int11)*(1-mean_int((alphadeltahatdiff_chat)/mean_h_int11,y11,mean_h_int11)))./density_int((alphadeltahatdiff_chat)/density_h_int,density_h_int);
			
			asymb_covar_int = (C*invd2dgdg)*Omega*(invd2dgdg*C');
			asymb_var_alpha1 = asymb_covar_int(1,1); 
			conf_alpha1 = alphahat(1) + norminv(1-alpha_test/2,0,1)*[-sqrt(asymb_var_alpha1/(N*mean_h_int00^8)),sqrt(asymb_var_alpha1/(N*mean_h_int00^8))];
			asymb_var_alpha2 = asymb_covar_int(2,2); 
			conf_alpha2 = alphahat(2) + norminv(1-alpha_test/2,0,1)*[-sqrt(asymb_var_alpha2/(N*mean_h_int00^8)),sqrt(asymb_var_alpha2/(N*mean_h_int00^8))];
			asymb_var_delta1 = asymb_covar_int(3,3);
			conf_delta1 = deltahat(1) + norminv(1-alpha_test/2,0,1)*[-sqrt(asymb_var_delta1/(N*mean_h_int00^8)),sqrt(asymb_var_delta1/(N*mean_h_int00^8))];
			asymb_var_delta2 = asymb_covar_int(4,4);
			conf_delta2 = deltahat(2) + norminv(1-alpha_test/2,0,1)*[-sqrt(asymb_var_delta2/(N*mean_h_int00^8)),sqrt(asymb_var_delta2/(N*mean_h_int00^8))];
		
       		else
			% Return NaN if not estimating the asymptotic approximation based confidence interval
			conf_alpha1 = [NaN NaN];
			conf_alpha2 = [NaN NaN];
			conf_delta1 = [NaN NaN];	
			conf_delta2 = [NaN NaN];	
		
		end;		

end


function density_int = density_int(x,h_int)
	N = size(x,1);
	
	%% Fourth order kernels
		K_eval1 = K(x(:,1));
		K_eval2 = K(x(:,2));
		K_int_eval = K_eval1.*K_eval2;
			
		Kx_int_eval = 1/h_int^2*1/N*ones(1,N)*K_int_eval;
		
	density_int = Kx_int_eval;

end

function mean_int = mean_int(x,y,h_int)
	N = size(x,1);

	%% Fourth order kernels
		K_eval1 = K(x(:,1));
		K_eval2 = K(x(:,2));
		K_int_eval = K_eval1.*K_eval2;

		Kx_int_eval = ones(1,N)*K_int_eval;
		Ky_int_eval = ones(1,N)*(K_int_eval.*y);
		
	mean_int = Ky_int_eval/Kx_int_eval;

end

function second_deriv_int = second_deriv_int(x,y,h_int)
	N = size(x,1);
	h_int = 1; %drops out of maximization problem, use for numerical precision

	%% Fourth order kernels
		K_eval1 = K(x(:,1));
		K_eval2 = K(x(:,2));
		K_int_eval = K_eval1.*K_eval2;

		dK_eval1 = dK(x(:,1));
		dK_eval2 = dK(x(:,2));
		dK1_int_eval = dK_eval1.*K_eval2;
		dK2_int_eval = K_eval1.*dK_eval2;

		dK2_eval1 = dK2(x(:,1));
		dK2_eval2 = dK2(x(:,2));
		dK12_int_eval = dK_eval1.*dK_eval2;
		dK11_int_eval = dK2_eval1.*K_eval2;
		dK22_int_eval = K_eval1.*dK2_eval2;

		Kx_int_eval = 1/h_int^2*1/N*ones(1,N)*K_int_eval;
		dKx1_int_eval = 1/h_int^3*1/N*ones(1,N)*dK1_int_eval;
		dKx2_int_eval = 1/h_int^3*1/N*ones(1,N)*dK2_int_eval;
		dKx12_int_eval = 1/h_int^4*1/N*ones(1,N)*dK12_int_eval;

		Ky_int_eval = 1/h_int^2*1/N*ones(1,N)*(K_int_eval.*y);
		dKy1_int_eval = 1/h_int^3*1/N*ones(1,N)*(dK1_int_eval.*y);
		dKy2_int_eval = 1/h_int^3*1/N*ones(1,N)*(dK2_int_eval.*y);
		dKy12_int_eval = 1/h_int^4*1/N*ones(1,N)*(dK12_int_eval.*y);

	second_deriv_int = max (( (dKy12_int_eval*Kx_int_eval+dKy1_int_eval*dKx2_int_eval-dKy2_int_eval*dKx1_int_eval-Ky_int_eval*dKx12_int_eval)*(Kx_int_eval)^2 - (dKy1_int_eval*Kx_int_eval-Ky_int_eval*dKx1_int_eval)*(2*dKx2_int_eval*Kx_int_eval)  ) / (Kx_int_eval)^4, 0);
end

function fourth_deriv_int = fourth_deriv_int(x,y,h_int,d1,d2,d3,d4)
	N = size(x,1);

	%% Fourth order kernels		
		K_eval1 = K(x(:,1));
		K_eval2 = K(x(:,2));
		K_int_eval = K_eval1.*K_eval2;

		dK_eval1 = dK(x(:,1));
		dK_eval2 = dK(x(:,2));
		dK1_int_eval = dK_eval1.*K_eval2;
		dK2_int_eval = K_eval1.*dK_eval2;

		dK2_eval1 = dK2(x(:,1));
		dK2_eval2 = dK2(x(:,2));
		dK12_int_eval = dK_eval1.*dK_eval2;
		dK11_int_eval = dK2_eval1.*K_eval2;
		dK22_int_eval = K_eval1.*dK2_eval2;
		
		dK3_eval1 = dK3(x(:,1));
		dK3_eval2 = dK3(x(:,2));		
		dK121_int_eval = dK2_eval1.*dK_eval2;
		dK122_int_eval = dK_eval1.*dK2_eval2;
		dK111_int_eval = dK3_eval1.*K_eval2;
		dK222_int_eval = K_eval1.*dK3_eval2;
				
		dK4_eval1 = dK4(x(:,1));
		dK4_eval2 = dK4(x(:,2));
		dK1211_int_eval = dK3_eval1.*dK_eval2;
		dK1212_int_eval = dK2_eval1.*dK2_eval2;
		dK1222_int_eval = dK_eval1.*dK3_eval2;
		dK1111_int_eval = dK4_eval1.*K_eval2;
		dK2222_int_eval = K_eval1.*dK4_eval2;

		Kx_int_eval = 1/h_int^2*1/N*ones(1,N)*K_int_eval;
		
		dKx1_int_eval = 1/h_int^3*1/N*ones(1,N)*dK1_int_eval;
		dKx2_int_eval = 1/h_int^3*1/N*ones(1,N)*dK2_int_eval;
		
		dKx12_int_eval = 1/h_int^4*1/N*ones(1,N)*dK12_int_eval;
		dKx11_int_eval = 1/h_int^4*1/N*ones(1,N)*dK11_int_eval;
		dKx22_int_eval = 1/h_int^4*1/N*ones(1,N)*dK22_int_eval;
		
		dKx121_int_eval = 1/h_int^5*1/N*ones(1,N)*dK121_int_eval;
		dKx122_int_eval = 1/h_int^5*1/N*ones(1,N)*dK122_int_eval;
		dKx111_int_eval = 1/h_int^5*1/N*ones(1,N)*dK111_int_eval;
		dKx222_int_eval = 1/h_int^5*1/N*ones(1,N)*dK222_int_eval;
		
		dKx_second(1:2,1:2) = NaN;
		dKx_third(1:2,1:2,1:2) = NaN;

		Kx = Kx_int_eval;

		dKx_first(1) = dKx1_int_eval;
		dKx_first(2) = dKx2_int_eval;
		
		dKx_second(1,2) = dKx12_int_eval;
		dKx_second(1,1) = dKx11_int_eval;
		dKx_second(2,2) = dKx22_int_eval;
		dKx_second(2,1) = dKx12_int_eval;
		
		dKx_third(1,2,1) = dKx121_int_eval;
		dKx_third(1,2,2) = dKx122_int_eval;
		dKx_third(1,1,1) = dKx111_int_eval;
		dKx_third(2,2,2) = dKx222_int_eval;
		dKx_third(2,1,1) = dKx121_int_eval;
		dKx_third(1,1,2) = dKx121_int_eval;
		dKx_third(2,1,2) = dKx122_int_eval;
		dKx_third(2,2,1) = dKx122_int_eval;
		
		Ky_int_eval = 1/h_int^2*1/N*ones(1,N)*(K_int_eval.*y);
		
		dKy1_int_eval = 1/h_int^3*1/N*ones(1,N)*(dK1_int_eval.*y);
		dKy2_int_eval = 1/h_int^3*1/N*ones(1,N)*(dK2_int_eval.*y);
		
		dKy12_int_eval = 1/h_int^4*1/N*ones(1,N)*(dK12_int_eval.*y);
		dKy11_int_eval = 1/h_int^4*1/N*ones(1,N)*(dK11_int_eval.*y);
		dKy22_int_eval = 1/h_int^4*1/N*ones(1,N)*(dK22_int_eval.*y);
		
		dKy121_int_eval = 1/h_int^5*1/N*ones(1,N)*(dK121_int_eval.*y);
		dKy122_int_eval = 1/h_int^5*1/N*ones(1,N)*(dK122_int_eval.*y);
		dKy111_int_eval = 1/h_int^5*1/N*ones(1,N)*(dK111_int_eval.*y);
		dKy222_int_eval = 1/h_int^5*1/N*ones(1,N)*(dK222_int_eval.*y);
		
		dKy1211_int_eval = 1/h_int^6*1/N*ones(1,N)*(dK1211_int_eval.*y);
		dKy1212_int_eval = 1/h_int^6*1/N*ones(1,N)*(dK1212_int_eval.*y);
		dKy1222_int_eval = 1/h_int^6*1/N*ones(1,N)*(dK1222_int_eval.*y);
		dKy1111_int_eval = 1/h_int^6*1/N*ones(1,N)*(dK1111_int_eval.*y);
		dKy2222_int_eval = 1/h_int^6*1/N*ones(1,N)*(dK2222_int_eval.*y);
		
		dKy_second(1:2,1:2) = NaN;
		dKy_third(1:2,1:2,1:2) = NaN;
		dKy_fourth(1:2,1:2,1:2,1:2) = NaN;
		
		Ky = Ky_int_eval;

		dKy_first(1) = dKy1_int_eval;
		dKy_first(2) = dKy2_int_eval;
		
		dKy_second(1,2) = dKy12_int_eval;
		dKy_second(1,1) = dKy11_int_eval;
		dKy_second(2,2) = dKy22_int_eval;
		dKy_second(2,1) = dKy12_int_eval;
		
		dKy_third(1,2,1) = dKy121_int_eval;
		dKy_third(1,2,2) = dKy122_int_eval;
		dKy_third(1,1,1) = dKy111_int_eval;
		dKy_third(2,2,2) = dKy222_int_eval;
		dKy_third(2,1,1) = dKy121_int_eval;
		dKy_third(1,1,2) = dKy121_int_eval;
		dKy_third(2,1,2) = dKy122_int_eval;
		dKy_third(2,2,1) = dKy122_int_eval;
		
		dKy_fourth(1,2,1,1) = dKy1211_int_eval;
		dKy_fourth(1,2,1,2) = dKy1212_int_eval;
		dKy_fourth(1,2,2,2) = dKy1222_int_eval;
		dKy_fourth(1,1,1,1) = dKy1111_int_eval;
		dKy_fourth(2,2,2,2) = dKy2222_int_eval;
		dKy_fourth(2,1,1,1) = dKy1211_int_eval;
		dKy_fourth(1,1,2,1) = dKy1211_int_eval;
		dKy_fourth(1,1,1,2) = dKy1211_int_eval;
		
		dKy_fourth(2,2,1,1) = dKy1212_int_eval;
		dKy_fourth(1,2,2,1) = dKy1212_int_eval;
		dKy_fourth(2,1,1,2) = dKy1212_int_eval;
		dKy_fourth(1,1,2,2) = dKy1212_int_eval;
		dKy_fourth(2,1,2,1) = dKy1212_int_eval;
		
		dKy_fourth(2,1,2,2) = dKy1222_int_eval;
		dKy_fourth(2,2,1,2) = dKy1222_int_eval;
		dKy_fourth(2,2,2,1) = dKy1222_int_eval;	
		
	a(1:2) = 0;
	for i=1:2,
		a(i) = dKy_first(i)*Kx - Ky*dKx_first(i);
	end;

	a_first(1:2,1:2) = 0;
	b(1:2,1:2) = 0;
	for i=1:2,
		for j=1:2,
			a_first(i,j) = dKy_second(i,j)*Kx + dKy_first(i)*dKx_first(j) - dKy_first(j)*dKx_first(i) - Ky*dKx_second(i,j);
			b(i,j) = a_first(i,j)*Kx^2 - a(i)*2*Kx*dKx_first(j);
		end;
	end;

	a_second(1:2,1:2,1:2) = 0;
	b_first(1:2,1:2,1:2) = 0;
	c(1:2,1:2,1:2) = 0;
	for i=1:2,
		for j=1:2,
			for k=1:2,
				a_second(i,j,k) = dKy_third(i,j,k)*Kx + dKy_second(i,j)*dKx_first(k) + dKy_second(i,k)*dKx_first(j) + dKy_first(i)*dKx_second(j,k) - dKy_second(j,k)*dKx_first(i) - dKy_first(j)*dKx_second(i,k) - dKy_first(k)*dKx_second(i,j) - Ky*dKx_third(i,j,k);
				b_first(i,j,k) = a_second(i,j,k)*Kx^2 + a_first(i,j)*2*Kx*dKx_first(k) - a_first(i,k)*2*Kx*dKx_first(j) - a(i)*(2*dKx_first(k)*dKx_first(j)+2*Kx*dKx_second(j,k));
				c(i,j,k) = b_first(i,j,k)*Kx^4 - b(i,j)*4*Kx^3*dKx_first(k);	
			end;
		end;
	end;

	a_third(1:2,1:2,1:2,1:2) = 0;
	b_second(1:2,1:2,1:2,1:2) = 0;
	c_first(1:2,1:2,1:2,1:2) = 0;
	f(1:2,1:2,1:2,1:2) = 0;
	for i=1:2,
		for j=1:2,
			for k=1:2,
				for l=1:2,
					a_third(i,j,k,l) = dKy_fourth(i,j,k,l)*Kx + dKy_third(i,j,k)*dKx_first(l) + dKy_third(i,j,l)*dKx_first(k) + dKy_second(i,j)*dKx_second(k,l) + dKy_third(i,k,l)*dKx_first(j) + dKy_second(i,k)*dKx_second(j,l) + dKy_second(i,l)*dKx_second(j,k) + dKy_first(i)*dKx_third(j,k,l) - dKy_third(j,k,l)*dKx_first(i) - dKy_second(j,k)*dKx_second(i,l) - dKy_second(j,l)*dKx_second(i,k) - dKy_first(j)*dKx_third(i,k,l) - dKy_second(k,l)*dKx_second(i,j) - dKy_first(k)*dKx_third(i,j,l) - dKy_first(l)*dKx_third(i,j,k) - Ky*dKy_fourth(i,j,k,l);
					b_second(i,j,k,l) = a_third(i,j,k,l)*Kx^2 + a_second(i,j,k)*2*Kx*dKx_first(l) + a_second(i,j,l)*2*Kx*dKx_first(k) + a_first(i,j)*2*(dKx_first(l)*dKx_first(k)+Kx*dKx_second(k,l))-a_second(i,k,l)*2*Kx*dKx_first(j)-a_first(i,k)*2*(dKx_first(l)*dKx_first(j) + Kx*dKx_second(j,l)) - a_first(i,l)*(2*dKx_first(k)*dKx_first(j)+2*Kx*dKx_second(j,k))-a(i)*(2*dKx_second(k,l)*dKx_first(j)+2*dKx_first(k)*dKx_second(j,l)+2*dKx_first(l)*dKx_second(j,k)+2*Kx*dKx_third(j,k,l));
					c_first(i,j,k,l) = b_second(i,j,k,l)*Kx^4 + b_first(i,j,k)*4*Kx^3*dKx_first(l)-b_first(i,j,l)*4*Kx^3*dKx_first(k)-b(i,j)*4*(3*Kx^2*dKx_first(l)*dKx_first(k)+Kx^3*dKx_second(k,l));
					f(i,j,k,l) = (c_first(i,j,k,l)*Kx^8-c(i,j,k)*8*Kx^7*dKx_first(l))/Kx^16;
				end;
			end;
		end;
	end;

	fourth_deriv_int = f(d1,d2,d3,d4);

end

function [psix expx sqx] = psi(x)
	sqx = x.^2;
	expx = exp(-sqx/2);
	psix = 1/sqrt(2*pi).*expx;
end

function psi1x = psi1(x,expx)
	if exist('expx','var'),
		psi1x = 1/sqrt(2*pi)*(-x).*expx;
	else
		psi1x = 1/sqrt(2*pi)*(-x).*exp(-x.^2/2);	
	end;
end

function psi2 = psi2(x)
	psi2 = 1/sqrt(2*pi)*(x.^2-1).*exp(-x.^2/2);
end

function psi3 = psi3(x)
	psi3 = 1/sqrt(2*pi)*(3*x-x.^3).*exp(-x.^2/2);
end

function psi4 = psi4(x)
	psi4 = 1/sqrt(2*pi)*(-6*x.^2 + x.^4 + 3).*exp(-x.^2/2);
end

function [K psix expx sqx] = K(x)
	[psix expx sqx] = psi(x);
	K = 1/2*(3-sqx).*psix;
end
	
function dK = dK(x,psix,expx,sqx)
	if exist('psix','var') && exist('expx','var') && exist('sqx','var'),
		psi1x = psi1(x,expx);
		dK = 1/2*(-2*(x.*psix)+(3-sqx).*psi1x);
	else
		psix = psi(x);
		psi1x = psi1(x);
		dK = 1/2*(-2*(x.*psix)+(3-x.^2).*psi1x);
	end;
end

function dK2 = dK2(x)
	dK2 = 1/2*(  -2*psi(x) - 4*x.*psi1(x) + (3-x.^2).* psi2(x) );
end

function dK3 = dK3(x)
	dK3 = 1/2*(  -6*psi1(x) - 6*x.*psi2(x) + (3-x.^2).*psi3(x) );
end

function dK4 = dK4(x)
	dK4 = 1/2*(-12*psi2(x) - 8*x.*psi3(x) + (3-x.^2).*psi4(x) );
end
